/*
  libscheme
  Copyright (c) 1994 Brent Benson
  All rights reserved.

  Permission is hereby granted, without written agreement and without
  license or royalty fees, to use, copy, modify, and distribute this
  software and its documentation for any purpose, provided that the
  above copyright notice and the following two paragraphs appear in
  all copies of this software.
 
  IN NO EVENT SHALL BRENT BENSON BE LIABLE TO ANY PARTY FOR DIRECT,
  INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
  OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF BRENT
  BENSON HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

  BRENT BENSON SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT
  NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
  FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER
  IS ON AN "AS IS" BASIS, AND BRENT BENSON HAS NO OBLIGATION TO
  PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
  MODIFICATIONS.
*/

#define GEN_BIN_COMP_PROT(name) \
static int name (Scheme_Object *n1, Scheme_Object *n2)

#define GEN_NARY_COMP(name, scheme_name, bin_name) \
static Scheme_Object * \
name (int argc, Scheme_Object *argv[]) \
{ \
  int i; \
  if (argc < 1) scheme_wrong_count(scheme_name, 1, -1, argc, argv); \
  if (argc == 1) if (!SCHEME_NUMBERP(argv[0])) \
   scheme_wrong_type(scheme_name, "number", -1, 0, argv); \
  for ( i=0 ; i<(argc-1) ; ++i ) \
      if (! bin_name(argv[i], argv[i+1])) \
          return (scheme_false); \
  return scheme_true; \
}

#define GEN_BIN_PROT(name) \
static Scheme_Object *name (Scheme_Object *n1, Scheme_Object *n2)

#define GEN_BIN_THING(rettype, name, scheme_name, iop, fop, bn_op, bn_get) \
static rettype \
name (Scheme_Object *n1, Scheme_Object *n2) \
{ \
  Small_Bignum s1, s2; \
  if (SCHEME_INTP(n1)) \
    { \
      if (SCHEME_INTP(n2)) \
	return (iop(SCHEME_INT_VAL(n1), SCHEME_INT_VAL(n2))); \
      if (SCHEME_DBLP(n2)) \
        return (fop(SCHEME_INT_VAL(n1), SCHEME_DBL_VAL(n2))); \
      if (SCHEME_BIGNUMP(n2)) \
        return bn_op(bn_get(scheme_make_small_bignum(SCHEME_INT_VAL(n1), \
						     &s1)), \
		     bn_get(n2)); \
      WRONG_TYPE(scheme_name, "number", n2); \
    } \
  else if (SCHEME_DBLP(n1)) \
    { \
      if (SCHEME_INTP(n2)) \
        return (fop(SCHEME_DBL_VAL(n1), SCHEME_INT_VAL(n2))); \
      if (SCHEME_DBLP(n2))\
        return (fop(SCHEME_DBL_VAL(n1), SCHEME_DBL_VAL(n2))); \
      if (SCHEME_BIGNUMP(n2)) \
        return fop(SCHEME_DBL_VAL(n1), scheme_bignum_to_float(n2)); \
      WRONG_TYPE(scheme_name, "number", n2); \
    } \
  else if (SCHEME_BIGNUMP(n1)) \
    { \
       if (SCHEME_INTP(n2)) \
	 return bn_op(bn_get(n1), \
		      bn_get(scheme_make_small_bignum(SCHEME_INT_VAL(n2), &s2))); \
       if (SCHEME_DBLP(n2)) \
	 return fop(scheme_bignum_to_float(n1), SCHEME_DBL_VAL(n2)); \
       if (SCHEME_BIGNUMP(n2)) \
         return bn_op(bn_get(n1), bn_get(n2)); \
       WRONG_TYPE(scheme_name, "number", n2); \
    } else \
       WRONG_TYPE(scheme_name, "number", n1); \
  return 0; \
}

#define GEN_IDENT(x) x

#define GEN_BIN_OP(name, scheme_name, iop, fop, bn_op, bn_get) \
  GEN_BIN_THING(Scheme_Object *, name, scheme_name, iop, fop, bn_op, bn_get)
#define GEN_BIN_COMP(name, scheme_name, op, bn_op) \
  GEN_BIN_THING(int, name, scheme_name, op, op, bn_op, GEN_IDENT)

#define GEN_BIN_INT_OP(name, scheme_name, op) \
static Scheme_Object * \
name (Scheme_Object *n1, Scheme_Object *n2) \
{ \
  if(!SCHEME_INTP(n1)) WRONG_TYPE(scheme_name, "integer", n1);\
  if(!SCHEME_INTP(n2)) WRONG_TYPE(scheme_name, "integer", n2);\
  return (scheme_make_integer (SCHEME_INT_VAL(n1) op SCHEME_INT_VAL(n2))); \
}

#define GEN_NARY_OP(name, scheme_name, bin_name, ident) \
static Scheme_Object * \
name (int argc, Scheme_Object *argv[]) \
{ \
  Scheme_Object *ret; \
  int i; \
  if (argc == 2) return bin_name(argv[0], argv[1]); \
  if (argc == 1) return bin_name (scheme_make_integer(ident), argv[0]); \
  if (!argc) return scheme_make_integer(ident); \
  ret = argv[0]; \
  for (i = 1 ; i<argc ; ++i ) \
    ret = bin_name (ret, argv[i]); \
  return (ret); \
}

#define GEN_TWOARY_OP(name, scheme_name, bin_name) \
static Scheme_Object * \
name (int argc, Scheme_Object *argv[]) \
{ \
  Scheme_Object *ret; \
  int i; \
  if (argc < 1) scheme_wrong_count(scheme_name, 1, -1, argc, argv); \
  if (argc == 2) return bin_name(argv[0], argv[1]); \
  if (argc == 1) if (SCHEME_NUMBERP(argv[0])) return argv[0]; \
            else scheme_wrong_type(scheme_name, "number", -1, 0, argv); \
  ret = argv[0]; \
  for ( i=1 ; i<argc ; ++i ) \
    { \
      ret = bin_name (ret, argv[i]); \
    } \
  return (ret); \
}

#define GEN_UNARY_OP(name, scheme_name, c_name) \
static Scheme_Object * \
name (int argc, Scheme_Object *argv[]) \
{ \
  if (argc != 1) scheme_wrong_count(#scheme_name, 1, 1, argc, argv); \
  if (SCHEME_INTP (argv[0])) \
    return (scheme_make_double (c_name (SCHEME_INT_VAL (argv[0])))); \
  else if (SCHEME_DBLP(argv[0]))\
    return (scheme_make_double (c_name (SCHEME_DBL_VAL (argv[0])))); \
  else if (SCHEME_BIGNUMP(argv[0])) \
    return (scheme_make_double (c_name (scheme_bignum_to_float(argv[0])))); \
  else { \
   scheme_wrong_type(#scheme_name, "number", 0, argc, argv); \
   return scheme_void; \
  } \
}

