
(define mred:welcome-message
  (string-append
   "Welcome to MrEd, Copyright (c) 1995 Matthew Flatt. Based on the following:"
   mred:newline-string
   " wxWindows (c) 1994 Artificial Intelligence"
   " Applications Institute, The University of Edinburgh."
   mred:newline-string
   " MzScheme (c) 1995 Matthew Flatt."
   mred:newline-string
   " libscheme (c) 1994 Brent Benson."
   mred:newline-string
   " conservative GC (c) 1988, 1989 Hans-J. Boehm, Alan J. Demers"
   " (c) 1991-1994 Xerox Corp."
   mred:newline-string
   " C++ GC extension by Jesse Hull and John Ellis"
   " (c) 1994 Xerox Corp."
   mred:newline-string
   "See the license agreement or"
   " http://www.cs.rice.edu/~mflatt/mred.html for more info."))		     

(define mred:scheme-mode-edit%
  (make-class mred:std-edit%
    (lambda args
      (apply-super-init args)
      ((ivar set-mode) (make-object mred:scheme-mode%)))))
(constant mred:scheme-mode-edit%)
    
(define mred:scheme-edit%
  (make-class mred:std-edit%
    (inherit 
     begin-edit-sequence
     end-edit-sequence
     set-position
     insert
     change-style
     last-position
     get-start-position
     get-end-position
     get-text
     get-character
     find-string
     set-mode
     get-style-list)
    (private 
      old-stdout
      old-stderr)
    
    (class
      [CACHE-TIME 1] 
      [CACHE-WRITE-COUNT 100]
      
      [normal-font wx:const-modern]
      [normal-delta '()]
      [display-delta (make-object wx:style-delta%
				  wx:const-change-weight
				  wx:const-bold)]
      [error-delta (make-object wx:style-delta%
				wx:const-change-style
				wx:const-slant)]
      
      [prompt-list (list (list->string (list #\newline #\>)))])
    
    (private
      [prompt-mode? #f]
      
      [timer-on? #f]
      [timer-writes 0])
    (public
      [auto-set-wrap? #t]
      [autoprompting? #f]
      [enable-autoprompt
       (opt-lambda ([v #t])
	 (set! autoprompting? v))]
      this-out
      this-err
      [generic-write
       (lambda (s style-func)
	 (if timer-on?
	     (atomic
	      (lambda ()
		(when timer-on?
		      (set! timer-writes (add1 timer-writes))
		      (when (> timer-writes CACHE-WRITE-COUNT)
			    (end-edit-sequence)
			    (begin-edit-sequence)
			    (set! timer-writes 0)))))
	     (begin
	       (set! timer-writes 0)
	       (set! timer-on? #t)
	       (begin-edit-sequence)
	       (thread 
		(lambda ()
		  (sleep CACHE-TIME)
		  (atomic
		   (lambda ()
		     (end-edit-sequence)
		     (set! timer-on? #f)))))))
	 (begin-edit-sequence)
	 (set-position (last-position))
	 (if prompt-mode?
	     (insert #\newline))
	 (let ((start (last-position)))
	   (insert s)
	   (let ((end (last-position)))
	     (change-style () start end)
	     (style-func start end)))
	 (end-edit-sequence)
	 (set! prompt-mode? #f))]
      [this-out-write
       (lambda (s)
	 (generic-write s
			(lambda (start end)
			  (change-style display-delta 
					 start end))))]
      [this-err-write
       (lambda (s)
	 (generic-write s
			(lambda (start end)
			  (change-style error-delta start end))))]
      [generic-close (lambda () '())]
      
      [check-copy
       (lambda (start end str)
	 (let ((last (last-position)))
	   (if (< end last)
	       (if (let loop ([pos end])
		     (cond
		      [(= pos last) #f]
		      [(char-whitespace? (get-character pos))
		       (loop (add1 pos))]
		      [else #t]))
		   (let ((str
			  (list->string
			   (let loop ((str (string->list str)))
			     (cond
			      ((null? str) str)
			      ((char-whitespace? (car str))
			       (loop (cdr str)))
			      (else str))))))
		     (begin-edit-sequence)
		     (set-position last)
		     (insert str)
		     (let ((new-last (last-position)))
		       (insert #\newline)
		       (change-style () last new-last)
		       (end-edit-sequence)))
		   (insert #\newline end last))
	       (insert #\newline))))]
      
      [do-eval
       (lambda (str)
	 (set! prompt-mode? #f)
	 (mred:show-busy-cursor
	  (lambda ()
	    (eval-and-display str)))
	 (insert-prompt))])
    
    (public
      [eval-and-display
       (lambda (str)
	 (catch
	     escape
	   (catch-error
	    (lambda (short-escape dbg s . objs)
	      (display s this-err)
	      (map (lambda (obj)
		     (write obj this-err))
		   objs)
	      (display #\newline this-err)
	      (escape #f))
	    (dynamic-enable-break
	     (lambda ()
	       (let ([v (eval-string str #f)])
		 (cond
		   [(void? v) v]
		   [(and (is-a? v wx:snip%)
			 (not (send v is-owned?)))
		    (insert v)]
		   [else (pretty-print v this-out)])))))))])
    
    (public
      [find-prompt
       (lambda ()
	 (apply max (map (lambda (str)
			   (find-string str -1))
			 prompt-list)))]
      
      [on-local-char
       (lambda (key)
	 (let ((code (send key get-key-code)))
	   (if (or (= code 13) (= code 10))
	       (let ((start (get-start-position))
		     (end (get-end-position))
		     (last (last-position)))
		 (if (< start end)
		     (let ((str (get-text start end)))
		       (set-position end end)
		       (check-copy start end str)
		       (do-eval str))
		     (if (and (= start last) (not prompt-mode?)
			      autoprompting?)
			 (insert-prompt)
			 (let* ((pos (find-prompt))
				(balance (mred:scheme-balanced?
					  this
					  pos
					  start)))
			   (if balance
			       (let ((str (get-text pos end)))
				 (check-copy end end str)
				 (do-eval str))
			       ((ivar super on-local-char) key))))))
	       ((ivar super on-local-char) key))))]
      
      [takeover-output
       (lambda ()
	 (if (void? old-stdout)
	     (begin
	       (set! old-stdout (current-output-port))
	       (set! old-stderr (current-error-port))
	       (current-output-port this-out)
	       (current-error-port this-err))))]
      [release-output
       (lambda ()
	 (if (not (void? old-stdout))
	     (begin
	       (current-output-port old-stdout)
	       (current-error-port old-stderr)
	       (set! old-stdout (void))
	       (set! old-stderr (void)))))]
      
      [insert-prompt
       (lambda ()
	 (set! prompt-mode? #t)
	 (let* ((last (last-position))
		(last-str (get-text (- last 1) last)))
	   (begin-edit-sequence)
	   (if (not (string=? last-str mred:newline-string))
	       (insert #\newline))
	   (insert "> ")
	   (let ((last (last-position)))
	     (change-style normal-delta (- last 2) last))
	   (end-edit-sequence)))])
    (lambda p
      (apply-super-init p)
      (set-mode (make-object mred:scheme-interaction-mode%))
      (set! this-out (make-output-port this-out-write 
				       generic-close))
      (set! this-err (make-output-port this-err-write 
				       generic-close)))))
(constant mred:scheme-edit%)

(define mred:console-frame%
  (let-enumerate
      ([FILE-LOAD
	FILE-EDIT 
	FILE-NEW
	FILE-NEW-FRAME 
	FILE-SAVE 
	FILE-SAVE-AS
	FILE-CLOSE
	FILE-QUIT
	FILE-END-OF-MENU])
    (make-class mred:simple-frame%
      (inherit edit canvas show)
      (class
	[canvas%
	 (make-class mred:std-canvas%
	   (public
	     [edit% mred:scheme-edit%]))]
	[menu% mred:menu%]
	[menu-bar% mred:menu-bar%])
      (private 
	edit-offset 
	other-offset)
      (public 
	[next-menu-id
	 (lambda ()
	   other-offset)]
	[on-menu-command
	 ; Handle menu selections
	 (lambda (op)
	   (cond
	     ((> op edit-offset)
	      (send edit do-edit (- op edit-offset)))
	     ((= op FILE-EDIT) (mred:open-file))
	     ((= op FILE-NEW)
	      (let ([file (mred:put-file)])
		(if file
		    (mred:edit-file file))))
	     ((= op FILE-LOAD)
	      (let ((file (mred:get-file)))
		(if file
		    (load file))))
	     ((= op FILE-NEW-FRAME) (mred:edit-file #f))
	     ((= op FILE-SAVE) (send edit save-file
				     (send edit get-filename)))
	     ((= op FILE-SAVE-AS) (send edit save-file ""))
	     ((= op FILE-CLOSE) 
	      (when ((ivar on-close))
		(send edit release-output)
		(show #f)))
	     ((= op FILE-QUIT) (mred:exit))))])
      (opt-lambda ([close? #f][mssg mred:welcome-message])
	(super-init "MrEdConsole")
	
	; Build the menus:
	(let ((file-menu (make-object menu%))
	      (edit-menu (make-object menu%)))
	  (send file-menu append FILE-LOAD "Load Scheme File...")
	  (send file-menu append-separator)
	  (send file-menu append FILE-EDIT "Edit File...")
	  (send file-menu append FILE-NEW "Edit New File...")
	  (send file-menu append-separator)
	  (send file-menu append FILE-NEW-FRAME "New Frame")
	  (send file-menu append-separator)
	  (send file-menu append FILE-SAVE "Save Transcript")
	  (send file-menu append FILE-SAVE-AS 
		"Save Transcript As...")
	  (send file-menu append-separator)
	  (if close?
	      (send file-menu append FILE-CLOSE "Close"))
	  (send file-menu append FILE-QUIT "Quit")
	  
	  ; Use the built-in menu item lists for the edit menu:
	  (set! edit-offset FILE-END-OF-MENU)
	  (set! other-offset
		(send edit append-edit-items edit-menu edit-offset))
	  
	  ; Build and install the menu bar:
	  (let ((menu-bar (make-object menu-bar%
				       (list file-menu edit-menu)
				       (list "File" "Edit"))))
	    ((ivar set-menu-bar) menu-bar)))
	
	; Welcome message and initial prompt:
	(when mssg
	  (send edit insert mssg)
	  
	  (let ([last (send edit last-position)]
		[delta (make-object wx:style-delta%
				    wx:const-change-family
				    wx:const-decorative)])
	    (send edit insert #\newline)
	    (send delta set-delta wx:const-change-size 10)
	    (send edit change-style delta 0 last))
	  (send edit enable-autoprompt)
	  (send edit takeover-output)
	  (send edit insert-prompt)
	  (send edit clear-undos))

	(send edit set-file-format wx:const-media-ff-std)
	
	(show #t)))))
(constant mred:console-frame%)
