/*
 * File:        edjr.cc
 * Purpose:     EdJr main file - demonstration of wxMedia classes
 * Author:      Matthew Flatt
 * Created:     1995
 * Copyright:   (c) 1995, Matthew Flatt
 */

#include "wx.h"
#include "wx_media.h"

class EdJrWindow : public wxFrame
{
  wxMediaCanvas *display;
  wxMediaEdit *media;

  int editOffset, fontOffset;

public:
  EdJrWindow(char *filename);
  ~EdJrWindow();

  void OnSize(int x, int y);
  void OnMenuCommand(int op);
};

#define MARGIN 2

class EdJrApp: public wxApp
{
  EdJrWindow *main;
  int counter;
public:
  EdJrApp();
  wxFrame *OnInit(void);
};

EdJrApp TheApp;

EdJrApp::EdJrApp()
{
  wx_class = "EdJr";
}

wxFrame *EdJrApp::OnInit(void)
{
  wxInitMedia();

  main = new EdJrWindow(NULL);

  counter = 0;

  return main;
}

enum {
  EdJrFILE_OPEN = 1,
  EdJrFILE_SAVE,
  EdJrFILE_SAVE_AS,
  EdJrFILE_RELOAD,
  EdJrFILE_QUIT,
  EdJrLAST_MENU_PLUS_ONE
  };

EdJrWindow::EdJrWindow(char *filename)
: wxFrame(NULL, "EdJr", -1, -1, 600, 800, wxDEFAULT_FRAME + wxSDI, "frame")
{
  wxMenuBar *menubar;
  wxMenu *file, *edit;

  display = new wxMediaCanvas(this);

  media = new wxMediaEdit();
  display->SetMedia(media);

  menubar = new wxMenuBar();

  file = new wxMenu();
  file->Append(EdJrFILE_OPEN, "Open");
  file->Append(EdJrFILE_SAVE, "Save");
  file->Append(EdJrFILE_SAVE_AS, "Save As");
  file->Append(EdJrFILE_RELOAD, "Reload");  
  file->AppendSeparator();
  file->Append(EdJrFILE_QUIT, "Close");
    
  editOffset = EdJrLAST_MENU_PLUS_ONE;

  edit = new wxMenu();
  fontOffset = media->AppendEditItems(edit, editOffset);
  edit->AppendSeparator();
  media->AppendFontItems(edit, fontOffset);
  
  menubar->Append(file, "File");
  menubar->Append(edit, "Edit");

  SetMenuBar(menubar);

  Show(TRUE);

  if (filename)
    media->LoadFile(filename);
}

EdJrWindow::~EdJrWindow()
{
  delete display;
}

void EdJrWindow::OnMenuCommand(int op)
{
  if (!display)
    return;

  if (op >= fontOffset)
    media->DoFont(op - fontOffset);
  else if (op >= editOffset)
    media->DoEdit(op - editOffset);
  else {
    switch(op) {
    case EdJrFILE_OPEN:
      media->LoadFile("");
      break;
    case EdJrFILE_SAVE:
      media->SaveFile();
      break;
    case EdJrFILE_SAVE_AS:
      media->SaveFile("");
      break;
    case EdJrFILE_RELOAD:
      media->LoadFile();
      break;
    case EdJrFILE_QUIT:
      delete this;
      break;
    }
  }
}

void EdJrWindow::OnSize(int x, int y)
{
  int top_h;

  GetClientSize(&x, &y);

  top_h = 0;

  if (display)
    display->SetSize(MARGIN, MARGIN + top_h, 
		     x - 2 * MARGIN, y - (2 * MARGIN + top_h));
}
