
#ifdef wx_xview
#define MEDIA_CANVAS_INTERNAL_SCROLLS
#endif

class wxStyleCanvas;
class wxStyleScrollbar;

const long
wxMCANVAS_NO_H_SCROLL = 0x1,
wxMCANVAS_NO_V_SCROLL = 0x2,
wxMCANVAS_HIDE_H_SCROLL = 0x4,
wxMCANVAS_HIDE_V_SCROLL = 0x8;

enum {
  wxEDIT_BUFFER =1,
  wxPASTEBOARD_BUFFER
};

#define STD_STYLE "Standard"

extern int wxmeCheckFormatAndVersion(void);

class wxMediaAdmin;
class wxKeymap;

class wxMediaBuffer : public wxObject
{
 protected:
  wxMediaAdmin *admin;

  wxKeymap *map;

  Bool ownCaret;
  wxSnip *caretSnip;

  wxStyleList *styleList;
  long notifyId;
  
  char *filename;  /* last loaded file, or NULL */
  Bool tempFilename; /* TRUE if filename is just a temporary name */

  Bool userLocked, modified;

  int maxUndos;

  wxChangeRecord **changes;
  int changes_start, changes_end;
  wxChangeRecord **redochanges;
  int redochanges_start, redochanges_end;
  wxList *intercepted;
  Bool undomode, redomode, interceptmode, noundomode;

  Bool DoWriteHeadersFooters(wxMediaStreamOut &f, Bool headers);
  Bool ReadHeadersFooters(wxMediaStreamIn &f, Bool headers);

  Bool ReadSnipsFromFile(wxMediaStreamIn &f);

  virtual Bool ReadInsert(wxSnip *snip) = 0;

  void FreeOldCopies(void);
  void InstallCopyBuffer(long time);
  void DoPaste(long time);
  virtual void InsertPasteSnip(wxSnip *snip, wxBufferData *) = 0;
  virtual void InsertPasteString(char *str) = 0;

  void AddUndo(wxChangeRecord *);
  void PerformUndos(wxChangeRecord **, int& start, int& end);
  void PerformUndoList(wxList *);
  void AppendUndo(wxChangeRecord *, wxChangeRecord **, int& start, int& end);

  Bool DoOwnCaret(Bool ownit);
  Bool DoSetCaretOwner(wxSnip *);

  virtual void SettingAdmin(wxMediaAdmin *);

 public:
  wxMediaBuffer();
  ~wxMediaBuffer();

  int bufferType;

  void SetAdmin(wxMediaAdmin *admin);
  wxMediaAdmin *GetAdmin(void);

  virtual void OnEvent(wxMouseEvent &event) = 0;
  virtual void OnChar(wxKeyEvent &event) = 0;
  virtual void Refresh(float localx, float localy, float w, float h, 
		       Bool show_caret) = 0;
  virtual void OwnCaret(Bool ownit) = 0;
  virtual void SizeCacheInvalid(void) = 0;
  virtual void GetExtent(float *w, float *h) = 0;

  virtual void OnDefaultEvent(wxMouseEvent &event) = 0;
  virtual void OnLocalEvent(wxMouseEvent &event);

  virtual void OnDefaultChar(wxKeyEvent &event) = 0;
  virtual void OnLocalChar(wxKeyEvent &event);
  
  /* Callbacks for the wxSnipAdmin: */
  virtual wxDC *GetDC();
  virtual void GetViewSize(float *h, float *w);
  virtual Bool ScrollTo(wxSnip *, float localx, float localy, 
			float w, float h, Bool refresh, int bias = 0) = 0;
  virtual void SetCaretOwner(wxSnip *) = 0;
  virtual void Resized(wxSnip *, Bool redraw_now) = 0;
  virtual void NeedsUpdate(wxSnip *, float localx, float localy, 
			   float w, float h) = 0;
  virtual Bool ReleaseSnip(wxSnip *) = 0;

  virtual float ScrollLineLocation(long line, float displayHeight) = 0;
  virtual long NumScrollLines(float displayHeight) = 0;
  virtual long FindScrollLine(float y, float displayHeight) = 0;

  virtual wxMediaBuffer *CopySelf(void) = 0;
  virtual float GetDescent(void) = 0;
  virtual float GetSpace(void) = 0;
  virtual char *GetFlattenedText() = 0;

  void GlobalToLocal(float *x, float *y);
  void LocalToGlobal(float *x, float *y);

  /* Edit & Font menus: */
  void DoEdit(int op, Bool recursive = TRUE, long time = 0);
  void DoFont(int op, Bool recursive = TRUE);
  virtual void ChangeStyle(wxStyleDelta *) = 0;

  virtual void Clear() = 0;
  virtual void Cut(Bool extend, long time) = 0;
  virtual void Copy(Bool extend, long time) = 0;
  virtual void Paste(long time) = 0;
  virtual void Kill(long time) = 0;

  virtual void Insert(wxSnip *) = 0;

  void InsertBox(int type = wxEDIT_BUFFER);
  void InsertImage(char *filename = NULL, long type = -1, 
		   Bool relative = FALSE);

  void Undo(void);
  void Redo(void);
  void ClearUndos(void);

  void SetMaxUndoHistory(int);
  int GetMaxUndoHistory();

  int AppendEditItems(wxMenu *edit, int idOffset = 0);
  int AppendFontItems(wxMenu *font, int idOffset = 0);

  virtual Bool GetSnipLocation(wxSnip *thesnip, 
			       float *x = NULL, float *y = NULL) = 0;

  virtual Bool WriteToFile(wxMediaStreamOut &) = 0;
  virtual Bool ReadFromFile(wxMediaStreamIn &) = 0;

  /* Override this to put more information in the file. Make
     sure that you follow the rules for extra header data. */
  virtual Bool ReadHeaderFromFile(wxMediaStreamIn &, char *headerName);
  virtual Bool ReadFooterFromFile(wxMediaStreamIn &, char *headerName);
  virtual Bool WriteHeadersToFile(wxMediaStreamOut &);
  virtual Bool WriteFootersToFile(wxMediaStreamOut &);
  /* Use these functions for adding custom header data: */
  Bool BeginWriteHeaderFooterToFile(wxMediaStreamOut &, char *headerName,
				    long *dataBuffer);
  Bool EndWriteHeaderFooterToFile(wxMediaStreamOut &, long dataBuffer);
  
  void SetKeymap(wxKeymap *keymap = NULL);
  wxKeymap *GetKeymap(void);
  wxStyleList *GetStyleList(void);
  virtual void SetStyleList(wxStyleList *styles);
  virtual void StyleHasChanged(wxStyle *style) = 0;

  /* For making a lot of changes to be displayed at once: */
  virtual void BeginEditSequence(Bool undoable = TRUE) = 0;
  virtual void EndEditSequence(void) = 0;

  virtual void SetMaxWidth(float w) = 0;
  virtual void SetMinWidth(float w) = 0;
  virtual float GetMaxWidth() = 0;
  virtual float GetMinWidth() = 0;
  virtual void SetMinHeight(float w) = 0;
  virtual void SetMaxHeight(float w) = 0;
  virtual float GetMinHeight() = 0;
  virtual float GetMaxHeight() = 0;

  virtual void OnPaint(Bool pre, wxDC *dc,
		       float, float, float, float, 
		       float dx, float dy,
		       Bool show_caret);
  virtual void InvalidateBitmapCache(float x=0.0, float y=0.0,
				     float w=-1.0, float h=-1.0) = 0;

  void Print(char *filename=NULL, Bool interactive=TRUE);
  virtual void PrintToDC(wxDC *dc) = 0;

  char *GetFilename(Bool *temp = NULL);
  virtual void SetFilename(char *, Bool temp = FALSE) = 0;
  
  virtual wxBufferData *GetSnipData(wxSnip *);
  virtual void SetSnipData(wxSnip *, wxBufferData *);

  virtual void OnChange(void) = 0;

  /* State */
  void Lock(Bool);
  Bool Modified(void);
  virtual void SetModified(Bool);

  void AddBufferFunctions(wxKeymap *tab);

 private:
  int numExtraHeaders;
};

Bool wxmbReadGlobalHeader(wxStyleList *, wxMediaStreamIn &f);
Bool wxmbReadGlobalFooter(wxMediaStreamIn &f);
Bool wxmbWriteGlobalHeader(wxStyleList *, wxMediaStreamOut &f);
Bool wxmbWriteGlobalFooter(wxMediaStreamOut &f);
wxStyle *wxmbStyleByIndex(int index);

Bool wxmbWriteSnipsToFile(wxMediaStreamOut &, wxStyleList *, 
			  wxList *, wxSnip *, wxSnip *, wxList *,
			  wxMediaBuffer *);

class wxDrawableMediaAdmin;
class SimpleScroll;

class wxMediaCanvas : public wxCanvas
{
  friend class wxDrawableMediaAdmin;

  wxDrawableMediaAdmin *admin;
  wxMediaBuffer *media;
  Bool noloop;
  long hpixelsPerScroll;
  int givenHScrollsPerPage;
  int hscrollsPerPage, vscrollsPerPage;
  int scrollHeight, scrollWidth;
  Bool focuson, focusforcedon;

  Bool allowXScroll, allowYScroll;
  Bool fakeXScroll, fakeYScroll;
  SimpleScroll *hscroll, *vscroll;
#ifdef MEDIA_CANVAS_INTERNAL_SCROLLS
  void PaintScrolls(void);
#endif

  void GetScroll(int *x, int *y);
  Bool ResetVisual(Bool reset_scroll = FALSE);

  void GetView(float *fx, float *fy, float *fh, float *fw, Bool full = FALSE);
  wxDC *GetDCAndOffset(float *fx, float *fy);
  void Redraw(float, float, float, float);
  Bool ScrollTo(float, float, float, float, Bool refresh = TRUE, int bias = 0);

 public:
  wxMediaCanvas(wxFrame *parent, int x = -1, int y = -1,
		int width = -1, int height = -1, char *name = "",
		long scrollStyle = 0,
		int scrollsPerPage = 100, wxMediaBuffer *m = NULL);
  ~wxMediaCanvas();

  Bool IsFocusOn();

  /* Override some wxCanvas methods: */
  virtual void OnSize(int, int);
  virtual void OnEvent(wxMouseEvent &event);
  virtual void OnChar(wxKeyEvent &event);
  virtual void OnPaint(void);
  virtual void OnScroll(wxCommandEvent& event);
  virtual void OnSetFocus();
  virtual void OnKillFocus();
  virtual void OnFocus(Bool focus);

  virtual void Scroll(int x, int y, Bool refresh);
  virtual void Scroll(int x, int y);

  void ForceDisplayFocus(Bool on);

  wxMediaBuffer *GetMedia(void);
  void SetMedia(wxMediaBuffer *, Bool redisplay = TRUE);
};

class wxCursor;

class wxMediaAdmin : public wxObject
{
 private:
  friend class wxMediaCanvas;
  friend class wxMediaEdit;
  friend class wxDrawableMediaAdmin;
  int standard; /* Used to recognize standard display. Hack. */
  
 public:
  wxMediaAdmin() {standard = 0;};

  /* Usually called by wxMediaBuffer objects: */
  virtual wxDC *GetDC(float *x = NULL, float *y = NULL) = 0;
  virtual void GetView(float *x, float *y, float *w, float *h, 
		       Bool full = FALSE) = 0;
  virtual Bool ScrollTo(float localx, float localy, float w, float h,
			Bool refresh = TRUE, int bias = 0) = 0;
  virtual void GrabCaret(void) = 0;

  virtual void Resized(Bool redraw_now) = 0;
  virtual void NeedsUpdate(float localx, float localy, float w, float h) = 0;

  virtual void SetCursor(wxCursor *cursor) = 0;
};

class wxStandardSnipAdmin : public wxSnipAdmin
{
  wxMediaBuffer *media;
 public:
  wxStandardSnipAdmin(wxMediaBuffer *m);

  wxMediaBuffer *GetMedia(void);

  wxDC *GetDC();
  void GetViewSize(float *h, float *w);
  Bool ScrollTo(wxSnip *, float localx, float localy, 
		float w, float h, Bool refresh, int bias = 0);
  void SetCaretOwner(wxSnip *);
  void Resized(wxSnip *, Bool redraw_now);
  void NeedsUpdate(wxSnip *, float localx, float localy, 
		   float w, float h);
  Bool ReleaseSnip(wxSnip *);

  void SetCursor(wxCursor *cursor);
};

class wxBufferData;
class wxStyleList;

extern wxList *wxmb_commonCopyBuffer;
extern wxList *wxmb_commonCopyBuffer2;
extern wxBufferData *wxmb_commonCopyRegionData;
extern wxStyleList *wxmb_copyStyleList;

void wxGetMediaPrintMargin(long *hm=NULL, long *vm=NULL);
void wxSetMediaPrintMargin(long hm=-1, long vm=-1);
