
class wxSnipAdmin;

/* The default is that none of these flags are on: */
enum {
  wxSNIP_IS_TEXT = 0x1,
  wxSNIP_CAN_APPEND = 0x2,
  wxSNIP_INVISIBLE = 0x4,
  wxSNIP_NEWLINE = 0x8, /* Soft newline, typically inserted by wxMediaEdit */
  wxSNIP_HARD_NEWLINE = 0x10, /* => Snip must be follwed by newline */
  wxSNIP_HANDLES_EVENTS = 0x20,
  wxSNIP_WIDTH_DEPENDS_ON_X = 0x40,
  wxSNIP_HEIGHT_DEPENDS_ON_Y = 0x80,
  wxSNIP_WIDTH_DEPENDS_ON_Y = 0x100,
  wxSNIP_HEIGHT_DEPENDS_ON_X = 0x200,
  wxSNIP_ANCHORED = 0x400,
  wxSNIP_USES_BUFFER_PATH = 0x800,
  wxSNIP_OWNED = 0x4000
};

#define WRITE_FUNC \
   Bool wxmbWriteSnipsToFile(class wxMediaStreamOut &, \
			     class wxStyleList *, \
			     class wxList *, class wxSnip *, \
			     class wxSnip *, wxList *, \
			     class wxMediaBuffer *)

class wxSnip;
extern WRITE_FUNC;

class wxSnipClass : public wxObject
{
  friend class wxStandardSnipClassList;
  friend class wxMediaBuffer;
  friend WRITE_FUNC;

  /* Special fields just for wxStandardSnipClassList & wxMediaEdit */
  Bool headerFlag;
  short mapPosition;

 public:
  char *classname;
  int version;

  /* If this flag is TRUE, then files saved to disk will be written
     assuming that, when the file is read from disk, the class will
     always be present. */
  Bool required;
  
  wxSnipClass();

  virtual class wxSnip *Read(wxMediaStreamIn &) = 0;

  virtual Bool ReadHeader(wxMediaStreamIn &);
  virtual void ReadDone(void);

  virtual Bool WriteHeader(wxMediaStreamOut &);
  virtual void WriteDone(void);
};

class wxSnipClassList : public /* should be private */ wxList
{
 public:
  wxSnipClassList(void);
  ~wxSnipClassList();

  wxSnipClass *Find(char *name);
  short FindPosition(wxSnipClass *);
  void Add(wxSnipClass *snipclass); /* Checks for duplicates */
  int Number(void);
  wxSnipClass *Nth(int);
};

enum {
  wxRESET_NO_MSG = 0,
  wxRESET_DONE_READ,
  wxRESET_DONE_WRITE
};

class wxStandardSnipClassList : public wxSnipClassList
{
 public:
  wxStandardSnipClassList(void);

  void ResetHeaderFlags(int doneMsg = wxRESET_NO_MSG);
  Bool Write(wxMediaStreamOut &f);
  Bool Read(wxMediaStreamIn &f);
  wxSnipClass *FindByMapPosition(short n);
};

extern wxStandardSnipClassList wxTheSnipClassList;

/******************************************************************/

class wxSnip : public wxObject
{
 private:
  friend class wxMediaEdit;
  friend class wxMediaPasteboard;
  friend class wxMediaBuffer;
  friend class wxMediaLine;
  friend WRITE_FUNC;

  /* For use only by the owning wxMediaBuffer */
  wxSnip *prev, *next;
  class wxMediaLine *line;

 protected:
  wxSnipAdmin *admin;

 public:
  long count;
  short flags;
  wxSnipClass *snipclass;
  wxStyle *style;

  wxSnip();
  ~wxSnip();

  wxSnip *Next(void);
  wxSnip *Previous(void);

  wxSnipAdmin *GetAdmin(void);
  virtual void SetAdmin(wxSnipAdmin *);

  Bool IsOwned(void);
  Bool ReleaseFromOwner(void);

  void SetCount(long count);
  void SetFlags(long flags);

  virtual void OnEvent(wxDC *dc, float x, float y, 
		       float mediax, float mediay, 
		       wxMouseEvent &event);
  virtual void OnChar(wxDC *dc, float x, float y, 
		      float mediax, float mediay, 
		      wxKeyEvent &event);
  virtual void OwnCaret(Bool);  
  
  virtual void DoEdit(int op, Bool recursive = TRUE, long time = 0);
  virtual void DoFont(int op, Bool recursive = TRUE);

  virtual int Match(wxSnip *other); 

  virtual void SizeCacheInvalid(void);

  virtual void GetExtent(wxDC *dc, 
			 float x, float y,
			 float *w = NULL, float *h = NULL, 
			 float *descent = NULL, float *space = NULL,
			 float *lspace = NULL, float *rspace = NULL);
  virtual float PartialOffset(wxDC *, float x, float y, long);
  virtual void Draw(wxDC *dc, float x, float y, 
		    float,float,float,float, 
		    float dx, float xy, 
		    Bool show_caret);
  virtual void Split(long position, wxSnip **first, wxSnip **second);
  virtual wxSnip *MergeWith(wxSnip *pred);
  virtual void GetText(char *s, long offset, long num);
  virtual char *GetText(long offset, long num, Bool flattened = FALSE);
  virtual wxSnip *Copy();
  virtual void Write(wxMediaStreamOut &f);

  virtual Bool Resize(float w, float h);

  virtual long GetNumScrollSteps();
  virtual long FindScrollStep(float y);
  virtual float GetScrollStepOffset(long i);

 protected:
  void Copy(wxSnip *); /* Copy basic values into given snip */
};

class wxTextSnip : public wxSnip
{
 protected:
  float w, h, descent, space;
  Bool contentsChanged;

 public:
  char *text;
  char *buffer;

  long allocated;

  wxTextSnip(long allocsize = 0); 
  ~wxTextSnip(); 

  virtual void SizeCacheInvalid(void);

  virtual void GetExtent(wxDC *dc,
			 float x, float y,
			 float *w = NULL, float *h = NULL, 
			 float *descent = NULL, float *space = NULL,
			 float *lspace = NULL, float *rspace = NULL);
  virtual float PartialOffset(wxDC *, float x, float y, long);
  virtual void Draw(wxDC *dc, float x, float y, 
		    float,float,float,float, 
		    float dx, float dy, Bool);
  virtual void Split(long position, wxSnip **first, wxSnip **second);
  virtual wxSnip *MergeWith(wxSnip *pred);

  virtual void Insert(char *str, long len, long pos = 0);
  virtual void Read(long len, wxMediaStreamIn &f);
  virtual void GetText(char *s, long offset, long num);
  virtual char *GetText(long offset, long num, Bool flattened = FALSE);
  virtual wxSnip *Copy(void);
  virtual void Write(wxMediaStreamOut &f);

 protected:
  void Copy(wxTextSnip *); /* Copy text values into given snip */
};

class wxTabSnip : public wxTextSnip
{
 public:
  wxTabSnip();
  
  virtual void GetExtent(wxDC *dc, 
			 float x, float y,
			 float *w = NULL, float *h = NULL, 
			 float *descent = NULL, float *space = NULL,
			 float *lspace = NULL, float *rspace = NULL);
  virtual float PartialOffset(wxDC *, float x, float y, long);
  virtual void Draw(wxDC *dc, float x, float y, 
		    float,float,float,float, 
		    float dx, float dy, Bool);
  virtual wxSnip *Copy();
};

class wxImageSnip : public wxSnip
{
 private:
  Bool fromfile;
  char *filename;
  long filetype;
  wxBitmap *bm;
  wxMemoryDC *offscreen;
  Bool relativePath;

 protected:
  float w, h, vieww, viewh, viewdx, viewdy;
  Bool contentsChanged;

 public:
  wxImageSnip(char *name = NULL, long type = 0, Bool relative = FALSE); 
  ~wxImageSnip(); 

  virtual void SizeCacheInvalid(void);

  virtual void GetExtent(wxDC *dc,
			 float x, float y,
			 float *w = NULL, float *h = NULL, 
			 float *descent = NULL, float *space = NULL,
			 float *lspace = NULL, float *rspace = NULL);

  virtual void Draw(wxDC *dc, float x, float y, 
		    float,float,float,float, 
		    float dx, float dy, Bool);

  virtual wxSnip *Copy(void);

  virtual void Write(wxMediaStreamOut &f);

  void LoadFile(char *name, long type, Bool relative = FALSE);
  char *GetFilename(Bool *relative);
  long GetFiletype();

  void SetBitmap(wxBitmap *);

  void SetOffset(float dx, float dy);
  virtual Bool Resize(float w, float h);

  virtual long GetNumScrollSteps();
  virtual long FindScrollStep(float y);
  virtual float GetScrollStepOffset(long i);

  virtual void SetAdmin(wxSnipAdmin *);

 protected:
  void Copy(wxImageSnip *);
};

class wxSnipAdmin : public wxObject
{
 public:
  virtual wxMediaBuffer *GetMedia(void) = 0;

  virtual wxDC *GetDC() = 0;
  virtual void GetViewSize(float *h, float *w) = 0;
  virtual Bool ScrollTo(wxSnip *, float localx, float localy, 
			float w, float h, Bool refresh, int bias = 0) = 0;
  virtual void SetCaretOwner(wxSnip *) = 0;
  virtual void Resized(wxSnip *, Bool redraw_now) = 0;
  virtual void NeedsUpdate(wxSnip *, float localx, float localy, 
			   float w, float h) = 0;
  virtual Bool ReleaseSnip(wxSnip *) = 0;

  virtual void SetCursor(class wxCursor *cursor) = 0;
};

#define wxMSNIPBOX_XMARGIN 5
#define wxMSNIPBOX_YMARGIN 5
#define wxMSNIPBOX_XINSET 1
#define wxMSNIPBOX_YINSET 0

class wxMediaSnip : public wxSnip
{
  friend class wxSnipMediaAdmin;

  wxMediaBuffer *me;
  class wxSnipMediaAdmin *myAdmin;

  Bool withBorder;
  int leftMargin, topMargin, rightMargin, bottomMargin;
  int leftInset, topInset, rightInset, bottomInset;

  float minWidth, maxWidth, minHeight, maxHeight;

 public:
  wxMediaSnip(wxMediaBuffer *useme = NULL,
	      Bool withBorder = TRUE,
	      int leftMargin = wxMSNIPBOX_XMARGIN,
	      int topMargin = wxMSNIPBOX_YMARGIN,
	      int rightMargin = wxMSNIPBOX_XMARGIN,
	      int bottomMargin = wxMSNIPBOX_YMARGIN,
	      int leftInset = wxMSNIPBOX_XINSET,
	      int topInset = wxMSNIPBOX_YINSET,
	      int rightInset = wxMSNIPBOX_XINSET,
	      int bottomInset = wxMSNIPBOX_YINSET,
	      float w = -1, float W = -1, 
	      float h = -1, float H = -1);
  ~wxMediaSnip();

  virtual void SetAdmin(wxSnipAdmin *a);

  virtual void OnEvent(wxDC *, float, float, float,float, wxMouseEvent &event);
  virtual void OnChar(wxDC *, float, float, float, float, wxKeyEvent &event);
  virtual void OwnCaret(Bool);  

  virtual void DoEdit(int op, Bool recursive = TRUE, long time = 0);
  virtual void DoFont(int op, Bool recursive = TRUE);

  virtual Bool Match(wxSnip *other); 

  virtual void SizeCacheInvalid(void);

  virtual char *GetText(long offset, long num, Bool flattened = FALSE);

  virtual void GetExtent(wxDC *dc, 
			 float x, float y,
			 float *w = NULL, float *h = NULL, 
			 float *descent = NULL, float *space = NULL,
			 float *lspace = NULL, float *rspace = NULL);
  virtual void Draw(wxDC *dc, float x, float y, 
		    float, float, float, float, float dx, float dy, 
		    Bool show_caret);
  virtual wxSnip *Copy(void);

  virtual void Write(wxMediaStreamOut &f);

  virtual long GetNumScrollSteps();
  virtual long FindScrollStep(float y);
  virtual float GetScrollStepOffset(long i);

  void SetMaxWidth(float);
  void SetMaxHeight(float);
  float GetMaxWidth(void);
  float GetMaxHeight(void);
  void SetMinWidth(float);
  void SetMinHeight(float);
  float GetMinWidth(void);
  float GetMinHeight(void);

  void ShowBorder(Bool show);
  Bool BorderVisible();

  void SetMargin(int lm, int tm, int rm, int bm);
  void GetMargin(int *lm, int *tm, int *rm, int *bm);
  void SetInset(int lm, int tm, int rm, int bm);
  void GetInset(int *li, int *ti, int *ri, int *bi);

  virtual Bool Resize(float w, float h);

  wxMediaBuffer *GetThisMedia(void);
};

/**********************************************************************/

class wxMediaStreamOut;
class wxBufferData;

class wxBufferDataClass : public wxObject
{
 private:
  friend class wxBufferDataClassList;
  friend class wxMediaBuffer;
  friend WRITE_FUNC;
  friend Bool wxmbWriteBufferData(wxMediaStreamOut&, wxBufferData *data);

  int mapPosition;
 public:
  char *classname;
  Bool required;

  wxBufferDataClass();

  virtual class wxBufferData *Read(wxMediaStreamIn &) = 0;
};

class wxBufferData : public wxObject
{
 public:
  wxBufferDataClass *dataclass;
  wxBufferData *next; /* Used to chain them in a list */

  wxBufferData();
  ~wxBufferData();

  virtual Bool Write(wxMediaStreamOut &) = 0;
};

class wxBufferDataClassList : public /* should be private */ wxList
{
 public:
  wxBufferDataClassList();
  ~wxBufferDataClassList();

  wxBufferDataClass *Find(char *name);
  short FindPosition(wxBufferDataClass *);
  void Add(wxBufferDataClass *dataclass); /* Checks for duplicates */
  int Number(void);
  wxBufferDataClass *Nth(int);

  Bool Write(wxMediaStreamOut &f);
  Bool Read(wxMediaStreamIn &f);
  wxBufferDataClass *FindByMapPosition(short n);
};

extern wxBufferDataClassList wxTheBufferDataClassList;

/**********************************************************************/

class wxLocationBufferData : public wxBufferData
{
 public:
  float x, y;

  wxLocationBufferData();

  Bool Write(wxMediaStreamOut &);
};
