
#include "wx.h"
#include "wxscheme.h"
#include "wx_media.h"

#include "wxs_obj.h"
#include "wxs_win.h"
#include "wxs_fram.h"
#include "wxs_gdi.h"
#include "wxs_item.h"
#include "wxs_butn.h"
#include "wxs_text.h"
#include "wxs_ckbx.h"
#include "wxs_chce.h"
#include "wxs_evnt.h"
#include "wxs_panl.h"
#include "wxs_list.h"
#include "wxs_medi.h"
#include "wxs_mede.h"
#include "wxs_madm.h"
#include "wxs_snip.h"
#include "wxs_mpb.h"
#include "wxs_mio.h"
#include "wxs_styl.h"
#include "wxs_dc.h"
#include "wxs_cnvs.h"
#include "wxs_menu.h"
#include "wxs_bmap.h"
#include "wxs_ipc.h"
#include "wxs_help.h"
#include "wxs_misc.h"
#include "wxs_rado.h"
#include "wxs_slid.h"
#include "wxs_lbox.h"
#include "wxs_cstr.h"
// #include "wxs_form.h"

#include "wxs_glob.h"

Scheme_Env *global_env;

char *WXSCHEME_wrong = "wrong number of parameters to method %s";
char *WXSCHEME_too_many = "too many parameters to method %s";
char *WXSCHEME_too_few = "too few parameters to method %s";
char *WXSCHEME_expected = "expected a %s in %s";

static void wxScheme_Install(Scheme_Env *global_env);

Scheme_Env *wxsScheme_setup(void)
{
  global_env = scheme_basic_env();
  objscheme_init(global_env);

  wxScheme_Install(global_env);

  return global_env;
}

static Scheme_Object *InstallIntoProcess(int argc, Scheme_Object *argv[])
{
  if (argc != 1)
    scheme_wrong_count("wx:install-into-process", 1, 1, argc, argv);
  if (!SCHEME_PROCESSP(argv[0]))
    scheme_wrong_type("wx:install-into-process", "process", 1, argc, argv);

  wxScheme_Install(((Scheme_Process *)argv[0])->env);

  return scheme_void;
}

static Scheme_Object *install = NULL;

static void wxScheme_Install(Scheme_Env *global_env)
{
  if (!install)
    install = scheme_make_prim(InstallIntoProcess);

  scheme_add_global_constant("wx:install-into-process", install, global_env);

  /* Order is important! Base class must be initialized before derived. */
  objscheme_setup_wxObject(global_env);
  objscheme_setup_wxNode(global_env);
  objscheme_setup_wxList(global_env);
  objscheme_setup_wxHashTable(global_env);
  objscheme_setup_wxPathList(global_env);
  objscheme_setup_wxStringList(global_env);
  objscheme_setup_wxWindow(global_env);
  objscheme_setup_wxFrame(global_env);
  objscheme_setup_wxColour(global_env);
  objscheme_setup_wxColourMap(global_env);
  objscheme_setup_wxColourDatabase(global_env);
  objscheme_setup_wxPoint(global_env);
  objscheme_setup_wxIntPoint(global_env);
  objscheme_setup_wxBrush(global_env);
  objscheme_setup_wxBrushList(global_env);
  objscheme_setup_wxPen(global_env);
  objscheme_setup_wxPenList(global_env);
  objscheme_setup_wxIcon(global_env);
  objscheme_setup_wxCursor(global_env);
  objscheme_setup_wxFont(global_env);
  objscheme_setup_wxFontList(global_env);
  objscheme_setup_wxFontNameDirectory(global_env);
  objscheme_setup_wxBitmap(global_env);
  objscheme_setup_wxItem(global_env);
  objscheme_setup_wxMessage(global_env);
  objscheme_setup_wxButton(global_env);
  objscheme_setup_wxRadioBox(global_env);
  objscheme_setup_wxCheckBox(global_env);
  objscheme_setup_wxListBox(global_env);
  objscheme_setup_wxChoice(global_env);
  objscheme_setup_wxSlider(global_env);
  objscheme_setup_wxText(global_env);
  objscheme_setup_wxMultiText(global_env);
  objscheme_setup_wxTextWindow(global_env);
  objscheme_setup_wxMenu(global_env);
  objscheme_setup_wxMenuBar(global_env);
  objscheme_setup_wxMenuItem(global_env);
  objscheme_setup_wxEvent(global_env);
  objscheme_setup_wxCommandEvent(global_env);
  objscheme_setup_wxKeyEvent(global_env);
  objscheme_setup_wxMouseEvent(global_env);
  objscheme_setup_wxPanel(global_env);
  objscheme_setup_wxDialogBox(global_env);
#if USE_ENHANCED_DIALOG
  objscheme_setup_wxEnhDialogBox(global_env);
#endif
//  objscheme_setup_wxFormItem(global_env);
//  objscheme_setup_wxForm(global_env);
  objscheme_setup_wxDC(global_env);
  objscheme_setup_wxCanvasDC(global_env);
  objscheme_setup_wxMemoryDC(global_env);
  objscheme_setup_wxPostScriptDC(global_env);
#ifdef wx_msw
  objscheme_setup_wxPrinterDC(global_env);
#endif
  objscheme_setup_wxCanvas(global_env);
  objscheme_setup_wxMediaGlobal(global_env);
  objscheme_setup_wxMediaCanvas(global_env);
  objscheme_setup_wxMediaBuffer(global_env);
  objscheme_setup_wxMediaEdit(global_env);
  objscheme_setup_wxMediaPasteboard(global_env);
  objscheme_setup_wxSnipClass(global_env);
  objscheme_setup_wxSnipClassList(global_env);
  objscheme_setup_wxSnip(global_env);
  objscheme_setup_wxTextSnip(global_env);
  objscheme_setup_wxTabSnip(global_env);
  objscheme_setup_wxImageSnip(global_env);
  objscheme_setup_wxMediaSnip(global_env);
  objscheme_setup_wxSnipAdmin(global_env);
  objscheme_setup_wxMediaAdmin(global_env);
  objscheme_setup_wxBufferData(global_env);
  objscheme_setup_wxBufferDataClass(global_env);
  objscheme_setup_wxBufferDataClassList(global_env);
  objscheme_setup_wxKeymap(global_env);
  objscheme_setup_wxMediaStreamInBase(global_env);
  objscheme_setup_wxMediaStreamOutBase(global_env);
  objscheme_setup_wxMediaStreamIn(global_env);
  objscheme_setup_wxMediaStreamOut(global_env);
  objscheme_setup_wxMediaWordbreakMap(global_env);
  objscheme_setup_wxMultColour(global_env);
  objscheme_setup_wxStyleDelta(global_env);
  objscheme_setup_wxStyle(global_env);
  objscheme_setup_wxStyleList(global_env);
  objscheme_setup_wxConnection(global_env);
  objscheme_setup_wxIPCObject(global_env);
  objscheme_setup_wxClient(global_env);
  objscheme_setup_wxServer(global_env);
  objscheme_setup_wxHelpInstance(global_env);
#if USE_METAFILE
  objscheme_setup_wxMetaFile(global_env);
  objscheme_setup_wxMetaFileDC(global_env);
#endif
  objscheme_setup_wxTimer(global_env);
  objscheme_setup_wxTypeTree(global_env);

  objscheme_setup_wxsGlobal(global_env);
  objscheme_setup_wxsIPCGlobal(global_env);
  objscheme_setup_wxsTypesGlobal(global_env);
//  objscheme_setup_wxFormGlobal(global_env);
  objscheme_setup_wxLayoutConstraints(global_env);
  objscheme_setup_wxIndividualLayoutConstraint(global_env);
}

void wxsScheme_rep(void)
{
  Scheme_Object *obj;

  do
    {
      printf ("> ");
      obj = scheme_read (scheme_stdin_port);
      if (obj == scheme_eof)
	{
	  printf ("\n; done\n");
	  exit (0);
	}
      obj = scheme_eval (obj, global_env);
      if (obj)
	{
	  scheme_write (obj, scheme_stdout_port);
	  printf ("\n");
	}
    }
  while ( 1 );
}
